;**************************************************************************
;*
;* FILE  server.asm
;* Copyright (c) 2005 Daniel Kahlin <daniel@kahlin.net>
;* Written by Daniel Kahlin <daniel@kahlin.net>
;* $Id: server.asm,v 1.32 2006-09-29 15:39:27 tlr Exp $
;*
;* DESCRIPTION
;*   Fast C64 DTV server.
;*
;******

.macro sir r
	.byte $42, r
.endmacro

.macro sac r
	.byte $32, r
.endmacro

	

.org $f7
mode_zp: .byte	$00
bank_zp: .byte	$00
ptr_zp: .word	$00
len_zp: .word	$00
tmp1_zp: .byte	$00
tmp2_zp: .byte	$00
chk_zp: .byte	$00


CMD_READ_MEM = $01
CMD_WRITE_MEM = $02
CMD_EXECUTE_MEM = $03

.reloc
.code

;**************************************************************************
;*
;* NAME  server
;*
;* DESCRIPTION
;*   Initialize for communication.
;*
;******
server:
; set up jumptable
	sta	store_port
	asl
	tax
	lda	porttab,x
	sta	ptr_zp
	lda	porttab+1,x
	sta	ptr_zp+1
	ldy	#jumptable_len-1
srv_lp2:
	lda	(ptr_zp),y
	sta	jumptable,y
	dey
	bpl	srv_lp2
; initialize
	jsr	init
server2:
	jsr	getmode
srv_lp1:
	lda	#0
	sta	$d020
	jsr	check_exit
	beq	srv_ex1
	lda	#11
	sta	$d020

	jsr	checkclk
	bne	srv_lp1
	
	jsr	getbyte
	cmp	#CMD_READ_MEM
	beq	srv_read_mem
	cmp	#CMD_WRITE_MEM
	beq	srv_write_mem
	cmp	#CMD_EXECUTE_MEM
	beq	srv_execute_mem

	jmp	srv_lp1
srv_ex1:
	jsr	uninit
	rts

srv_read_mem:
	lda	#5
	sta	$d020
	jsr	read_mem
	jmp	server2

srv_write_mem:
	lda	#5
	sta	$d020
	jsr	write_mem
	jmp	server2

srv_execute_mem:
	lda	#5
	sta	$d020
	jsr	execute_mem
	jmp	server2
	
check_exit:
	lda	$dc00
	tax
	and	#%01111111
	sta	$dc00
	lda	$dc01
	and	#%10000000	; run/stop
	stx	$dc00
	rts

getbyte:
	jmp	(jumptable)
putbyte:
	jmp	(jumptable+2)
getmode:
	jmp	(jumptable+4)
putmode:
	jmp	(jumptable+6)
checkclk:
	jmp	(jumptable+8)
restore:
	jmp	(jumptable+10)
	.IFNDEF IN_ROM
maketabs:
	jmp	(jumptable+12)
	.ENDIF ;IN_ROM
	
jumptable = $0120
	.IFDEF IN_ROM
jumptable_len = 12
	.ELSE
jumptable_len = 14
	.ENDIF
store_port = jumptable+jumptable_len

porttab:
	.WORD	tab_joy1,tab_joy2,tab_user0_4

tab_joy1:
	.WORD	getbyte_joy1
	.WORD	putbyte_joy1
	.WORD	getmode_joy1
	.WORD	putmode_joy1
	.WORD	checkclk_joy1
	.WORD	restore_joy1
	.IFNDEF IN_ROM
	.WORD	maketabs_joy1
	.ENDIF ;IN_ROM
tab_joy2:
	.WORD	getbyte_joy2
	.WORD	putbyte_joy2
	.WORD	getmode_joy2
	.WORD	putmode_joy2
	.WORD	checkclk_joy2
	.WORD	restore_joy2
	.IFNDEF IN_ROM
	.WORD	maketabs_joy1
	.ENDIF ;IN_ROM
tab_user0_4:
	.WORD	getbyte_user0_4
	.WORD	putbyte_user0_4
	.WORD	getmode_user0_4
	.WORD	putmode_user0_4
	.WORD	checkclk_user0_4
	.WORD	restore_user0_4
	.IFNDEF IN_ROM
	.WORD	maketabs_joy1
	.ENDIF ;IN_ROM

;**************************************************************************
;*
;* NAME  read_mem
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_READ_MEM ($01)
;*	01: mode    (0=ram, 1=rom)
;*	02: bank    ($00-$7f)
;*	03: ptr_lsb ($0000-$3fff)
;*	04: ptr_msb
;*	05: len_lsb ($0001-$4000)
;*	06: len_msb
;*   reply:
;*	00: databytes*len
;*	len: chksum 
;*
;*   the read access must be within the same 16Kb bank.
;*   
;******
read_mem:
	jsr	getbyte		; 0=ram, 1=rom
	tax
	lda	#%01010101	; ram
	cpx	#1
	bne	rm_skp3
	and	#%01000101	; Segment 2 = flash
rm_skp3:
	sta	mode_zp
	jsr	getbyte
	sta	bank_zp
	jsr	getbyte
	sta	ptr_zp
	jsr	getbyte
	clc
	adc	#$80		; Segment 2
	sta	ptr_zp+1
	jsr	getbyte
	sta	len_zp
	jsr	getbyte
	sta	len_zp+1

	lda	len_zp
	clc
	adc	ptr_zp
	sta	len_zp
	lda	len_zp+1
	adc	ptr_zp+1
	sta	len_zp+1

	sir	$e8
	ldy	bank_zp
	ldx	mode_zp
	sir	$12

	jsr	putmode

	lda	#0
	sta	chk_zp	
	ldx	#$05
	stx	$01
rm_lp1:	
	ldy	#0
	dec	$01
	lda	(ptr_zp),y
	inc	$01
	sta	$d020
	jsr	putbyte
	sec
	adc	chk_zp
	sta	chk_zp

	inc	ptr_zp
	bne	rm_skp1
	inc	ptr_zp+1
rm_skp1:
	lda	ptr_zp
	cmp	len_zp
	bne	rm_lp1
	lda	ptr_zp+1
	cmp	len_zp+1
	bne	rm_lp1

	ldx	#$07
	stx	$01

	lda	chk_zp	
	jsr	putbyte

	
	sir	$e8
	ldy	#$02
	ldx	#%01010101
	sir	$12

	jsr	getmode
	rts

;**************************************************************************
;*
;* NAME  write_mem
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_WRITE_MEM ($02)
;*	01: mode    (0=ram)
;*	02: bank    ($00-$7f)
;*	03: ptr_lsb ($0000-$3fff)
;*	04: ptr_msb
;*	05: len_lsb ($0001-$4000)
;*	06: len_msb
;*	07: databytes*len
;*	07+len: chksum 
;*   reply:
;*	00:  chksum 
;*
;*   the write access must be within the same 16Kb bank.
;*   
;* RETURNS: Z=0 if ok, Z=1 if checksum error.
;*   
;******
write_mem:
	jsr	getbyte		; 0=ram

	jsr	getbyte
	sta	bank_zp
	jsr	getbyte
	sta	ptr_zp
	jsr	getbyte
	clc
	adc	#$80		; Segment 2
	sta	ptr_zp+1
	jsr	getbyte
	sta	len_zp
	jsr	getbyte
	sta	len_zp+1

	lda	len_zp
	clc
	adc	ptr_zp
	sta	len_zp
	lda	len_zp+1
	adc	ptr_zp+1
	sta	len_zp+1

	sac	$ee
	lda	bank_zp
	sac	$00

	lda	#0
	sta	chk_zp	
	ldx	#$05
	stx	$01
wm_lp1:	
	jsr	getbyte
	ldy	#0
	dec	$01
	sta	(ptr_zp),y
	inc	$01
	sta	$d020
	sec
	adc	chk_zp
	sta	chk_zp

	inc	ptr_zp
	bne	wm_skp1
	inc	ptr_zp+1
wm_skp1:
	lda	ptr_zp
	cmp	len_zp
	bne	wm_lp1
	lda	ptr_zp+1
	cmp	len_zp+1
	bne	wm_lp1
	ldx	#$07
	stx	$01

;* get and check the checksum
	jsr	getbyte
	cmp	chk_zp		; check for error
	php
	
;* send back our view of the checksum
	jsr	putmode
	lda	chk_zp	
	jsr	putbyte

	sac	$ee
	lda	#2
	sac	$00

	jsr	getmode
	plp
	rts

;**************************************************************************
;*
;* NAME  execute_mem
;*
;* DESCRIPTION
;*   command:
;*	00: command CMD_EXECUTE_MEM ($03)
;*	01: ptr_lsb
;*	02: ptr_msb
;*
;******
exetramp_run = $110

execute_mem:
	ldx	#exetramp_end-exetramp-1
em_lp1:
	lda	exetramp,x
	sta	exetramp_run,x
	dex
	bpl	em_lp1

	jsr	getbyte
	sta	et_sm-exetramp+exetramp_run+1
	jsr	getbyte
	sta	et_sm-exetramp+exetramp_run+2

; clear up and run.
	jsr	uninit

	sir	$d8
	jmp	exetramp_run

exetramp:
	ldy	#$01
	ldx	#%01010101
	sir	$12
et_sm:
	jmp	et_sm
exetramp_end:
	
			
;**************************************************************************
;*
;* NAME  init
;*
;* DESCRIPTION
;*   Initialize for communication.
;*
;******
init:
	lda	$d020
	sta	store_d020

	ldx	#40
in_lp1:
	lda	text-1,x
	and	#$3f
	sta	$0400-1,x
	lda	#1
	sta	$d800-1,x
	dex
	bne	in_lp1

; Acc*12
	lda	store_port
	asl
	adc	store_port
	asl
	asl
	tay
in_lp2:
	lda	str,y
	and	#$3f
	sta	$0400+18,x
	iny
	inx	
	cpx	#12
	bne	in_lp2
	 		
	sir	$e8
	sty	store_reg_e
	stx	store_reg_8
	sir	$12

; enable extended features
	lda	#$01
	sta	$d03f
; disable badlines
	lda	#%00100000
	sta	$d03c

; burst enable, skip internal cycle
	sac	$99
	lda	#%00000011
	sac	$00

	.IFNDEF IN_ROM
	jsr	maketabs
	.ENDIF ;IN_ROM

	jsr	getmode
	rts

text:
	.BYTE	"DTVTRANS                      , EXIT=R/S" 
str:
	.BYTE	"$DC01 (JOY1)"
	.BYTE	"$DC00 (JOY2)"
	.BYTE	"$DD01 (USER)"
	
;**************************************************************************
;*
;* NAME  uninit
;*
;* DESCRIPTION
;*   Restore ports and banks.
;*
;******
uninit:	
; Default memory map.
	sir	$e8
	ldy	store_reg_e
	ldx	store_reg_8
	sir	$12

; all port bits as inputs, and wait for transmitter to tri-state
	jsr	restore

; disable burst, do not skip internal cycles anymore.
	sac	$99
	lda	#%00000000
	sac	$00

; enable badlines again
	lda	#%00000000
	sta	$d03c

; disable extended features
	lda	store_d020
	sta	$d020
	lda	#$00
	sta	$d03f

	rts


;**************************************************************************
;*
;* NAME  misc port stuff
;*
;* DESCRIPTION
;*
;******
	.macro checkclkmac addr
	lda	#%00001000
; Wait for CLK low
	bit	addr
	.endmacro

	.macro getmodemac addr
; JoyA0-JoyA3=input, JoyA4=output
	.IF	addr=$dc00
	lda	#%00010000 | %11100000
	.ELSE
	lda	#%00010000
	.ENDIF
	sta	addr+2
; all ones (tri-state)
	lda	#%11111111
	sta	addr
	.endmacro


	.macro	putmodemac addr
	.local pm_lp1
; JoyA3=input, JoyA0-JoyA2,JoyA4=output
	.IF	addr=$dc00
	lda	#%00010111 | %11100000
	.ELSE
	lda	#%00010111
	.ENDIF
	sta	addr+2
; all ones (tri-state)
	lda	#%11111111
	sta	addr
	ldx	#0
pm_lp1:
	dex
	bne	pm_lp1
	.endmacro

	.macro	restoremac addr
	.local uin_lp1
; all port bits as inputs.
	lda	#%00000000
	sta	addr+2
	lda	#%11111111
	sta	addr
	.IF	addr-$dd01
; wait for all port bits to go high before exiting
uin_lp1:	
	lda	addr
	and	#%00011111
	cmp	#%00011111
	bne	uin_lp1
	.ENDIF
	.IF	addr=$dc00
	lda	#%11111111
	sta	addr+2
	.ENDIF
	.endmacro

;**************************************************************************
;*
;* NAME  getbytemac
;*
;* DESCRIPTION
;*   Get a byte from the master.
;*
;******
	.macro	getbytemac addr
	.local gb_lp1, gb_lp2, gb_lp3, gb_lp4
	lda	#%00001000
; Wait for CLK low
gb_lp1:
	bit	addr
	bne	gb_lp1
; D2/D1/D0 contains databits 7/6/5
	ldx	addr
; ACK=0 by shifting CLK into ACK.
	asl	addr
	ldy	tab1,x
	sty	tmp1_zp
; Wait for CLK high
gb_lp2:
	bit	addr
	beq	gb_lp2
; D2/D1/D0 contains databits 4/3/2
	ldx	addr
; ACK=1 by shifting CLK into ACK.
	asl	addr
	ldy	tab2,x
	sty	tmp2_zp
; Wait for CLK low
gb_lp3:
	bit	addr
	bne	gb_lp3
; D1/D0 contains databits 1/0  D2 contains SYNC
	lda	addr
; ACK=0 by shifting CLK into ACK.
	asl	addr
	and	#%00000011
	ora	tmp1_zp
	ora	tmp2_zp
	tax
	lda	#%00001000
; Wait for CLK high
gb_lp4:
	bit	addr
	beq	gb_lp4
; ACK=1 by shifting CLK into ACK.
	asl	addr
	txa
	.endmacro

;**************************************************************************
;*
;* NAME  putbytemac
;*
;* DESCRIPTION
;*   Send a byte to the master.
;*
;******
	.macro putbytemac addr
	.local pb_lp1, pb_lp2, pb_lp3, pb_lp4
	tax
	lda	#%00001000
; Wait for CLK low
pb_lp1:
	bit	addr
	bne	pb_lp1
; databits 7/6/5 to D2/D1/D0, ACK=0
	ldy	tab3,x
	sty	addr

; Wait for CLK high
pb_lp2:
	bit	addr
	beq	pb_lp2

; databits 4/3/2 to D2/D1/D0, ACK=1
	ldy	tab4,x
	sty	addr
	
; Wait for CLK low
pb_lp3:
	bit	addr
	bne	pb_lp3

; databits 1/0 to D1/D0, SYNC to D2, ACK=0
	txa
	and	#%00000011
	sta	addr

	lda	#%00001000
; Wait for CLK high
pb_lp4:
	bit	addr
	beq	pb_lp4
; ACK=1 by shifting CLK into ACK.
	asl	addr
	txa
	.endmacro

checkclk_joy1:
	checkclkmac	$dc01
	rts
getmode_joy1:
	getmodemac	$dc01
	rts
putmode_joy1:
	putmodemac	$dc01
	rts
restore_joy1:
	restoremac	$dc01
	rts
getbyte_joy1:
	getbytemac	$dc01
	rts
putbyte_joy1:
	putbytemac	$dc01
	rts
checkclk_joy2:
	checkclkmac	$dc00
	rts
getmode_joy2:
	getmodemac	$dc00
	rts
putmode_joy2:
	putmodemac	$dc00
	rts
restore_joy2:
	restoremac	$dc00
	rts
getbyte_joy2:
	getbytemac	$dc00
	rts
putbyte_joy2:
	putbytemac	$dc00
	rts
checkclk_user0_4:
	checkclkmac	$dd01
	rts
getmode_user0_4:
	getmodemac	$dd01
	rts
putmode_user0_4:
	putmodemac	$dd01
	rts
restore_user0_4:
	restoremac	$dd01
	rts
getbyte_user0_4:
	getbytemac	$dd01
	rts
putbyte_user0_4:
	putbytemac	$dd01
	rts


	.IFDEF	IN_ROM
;**************************************************************************
;*
;* NAME  tables
;*
;* DESCRIPTION
;*   precalculated tables for rom use
;*
;******
tab1:
	incbin	"tables_joy1.bin"
tab2	equ	tab1+256
tab3	equ	tab2+256
tab4	equ	tab3+256
	.ELSE ;IN_ROM
;**************************************************************************
;*
;* NAME  maketabs
;*
;* DESCRIPTION
;*   create tables for fast and/or operations
;*
;******
maketabs_joy1:
	ldx	#0
mt_lp1:
	txa
	asl
	asl
	pha
	asl
	asl
	asl
; Acc=(X & 0x07) << 5
	sta	tab1,x
	pla
	and	#%00011100
; Acc=(X & 0x07) << 2
	sta	tab2,x

	txa
	lsr
	lsr
	pha
	lsr
	lsr
	lsr
; Acc=(X >> 5) & 0x07 	
	sta	tab3,x
	pla
	and	#%00000111
	ora	#%00010000
; Acc=(X >> 2) & 0x07 | 0x10
	sta	tab4,x

	inx	
	bne	mt_lp1
	rts				

	.segment	"bss"
;**************************************************************************
;*
;* SECTION  tables
;*
;* DESCRIPTION
;*   bss storage
;*
;******

store_reg_8: .byte $00
store_reg_e: .byte $00
store_d020: .byte $00

tab1:
	.res	256
tab2:
	.res	256
tab3:
	.res	256
tab4:
	.res	256
	.ENDIF ;IN_ROM
	
; eof
