/*************************************************************************
 *
 * FILE  membuffer.hpp
 * Copyright (c) 2005 Daniel Kahlin <daniel@kahlin.net>
 * Written by Daniel Kahlin <daniel@kahlin.net>
 * $Id: membuffer.hpp,v 1.5 2006-07-23 08:08:09 tlr Exp $
 *
 * DESCRIPTION 
 *
 ******/
#ifndef MEMBUFFER_HPP
#define MEMBUFFER_HPP

#include <iostream>
#include <string>
#include "util/types.hpp"

class Address
{
public:
    Address()
	: rom_(false),
	  start_(0)
    {}

    bool is_rom() const { return rom_; }
    uint32_t get_start() const { return start_; }
    void set_start(uint32_t start) { start_ = start; }
    void set_start(const char* str);

protected:
    bool rom_;
    uint32_t start_;
};

class AddressRange : public Address
{
public:
    AddressRange()
	: length_(0)
    {}

    void set_range(const char* str);

    uint32_t get_length() const { return length_; }
    void set_length(uint32_t length) { length_= length; }
    uint32_t get_end() const { return start_+length_; }

protected:
    uint32_t length_;
};

class MemBuffer : public AddressRange
{
public:
    MemBuffer()
	: buffer_(0), buffer_pck_(0)
    {}
    ~MemBuffer()
    {
	delete buffer_;
    }

    void set_range(const char* str)
    {
	AddressRange::set_range(str);
	set_length(AddressRange::get_length());
    }
    void set_length(uint32_t length) 
    { 
	delete buffer_;
	AddressRange::set_length(length);
	buffer_ = new uint8_t[length];
    }

	void pack(bool do_pack);
    uint8_t* get_buffer() const { return buffer_; }
    void read_file(const char* name, bool use_prg=true);
    void write_file(const char* name, bool use_prg=true) const;

	void load_by_function(int (*fct)(char * buf, int maxbytes ));
	
protected:
    uint8_t* buffer_;
	uint8_t* buffer_pck_;
	uint32_t pck_len_;
	
	inline void pk_put_(uint8_t data);

	void pk_pack_();
	void pk_store_();

};


/*
 * -p = prog format  (default)
 * addresses/ranges:
 *    $008000-$00a000 (ram)
 *    $1008000-$100a000 (rom)
 *    $008000,8192   (ram, 8192 bytes)
 *
 */
#endif /* MEMBUFFER_HPP */
/* eof */
