/*************************************************************************
 *
 * FILE  getopt.cpp
 * Copyright (c) 2004 Daniel Kahlin <daniel@kahlin.net>
 * Written by Daniel Kahlin <daniel@kahlin.net>
 * $Id: getopt.cpp,v 1.5 2005-12-18 09:55:59 tlr Exp $
 *
 * DESCRIPTION
 *   Template for C++ source files.
 *
 ******/

#include "getopt.hpp"

/*************************************************************************
 *
 * CLASS  Getopt
 *
 * DESCRIPTION
 *
 * KNOWN BUGS
 *   none
 *
 ******/
Getopt::Getopt(int &argc, char **&argv, const char* optstring)
    :argc_(argc), argv_(argv), optind_(1)
{
    set_optstring(optstring);
}

Getopt::~Getopt()
{
}

void Getopt::set_optstring(const char* optstring)
{
    optmap_.clear();
    char c;
    bool have_arg;
    while ((c=*optstring)) {
	++optstring;
	have_arg=false;
	if (*optstring==':') {
	    have_arg=true;
	    ++optstring;
	}
	optmap_.insert(std::make_pair<char,bool>(c,have_arg));
    }
}

Getopt::Opt Getopt::next_opt()
{
    const char* arg;
    while ((arg=next_arg())) {

	if (arg[0]=='-') {
	    char c=arg[1];
	    if (c=='-')
		return Opt(true,Opt::OK); // '--' found, end of arguments

	    // check if this is a valid option
	    std::map<char,bool>::const_iterator it=optmap_.find(c);
	    if (it==optmap_.end()) 
		return Opt(false,Opt::INVALID,c); // invalid option

	    if (it->second) {
		const char* param=next_arg();
		if (!param)
		    return Opt(false,Opt::MISSING,c); // missing argument
		return Opt(it->first, param);		
	    } else {
		return Opt(it->first, 0);
	    }
	} else {
	    // this was not an option, back up and exit.
	    --optind_;
	    return Opt(true,Opt::OK);
	}
    }

    // end of arguments
    return Opt(true,Opt::OK);
}

const char* Getopt::next_arg()
{
    const char* arg;
    if (optind_ >= argc_)
	return 0;
    arg=argv_[optind_];
    ++optind_;
    return arg;
}
/* eof */
